#include <stdlib.h>
#include "invsubs.h"
#define PI              3.1415926

#define rcmul(x,y)      (x.re*y.re + x.im*y.im)
#define icmul(x,y)      (x.im*y.re - x.re*y.im)
#define  rmul(x,y)      (x.re*y.re - x.im*y.im)
#define  imul(x,y)      (x.im*y.re + x.re*y.im)

#define ABSOLUTE(x) ( (x > 0) ? x : -x)
#define MAX(a,b) ( (a) > (b) ? (a) : (b) )
 
#define MINBLOB 10
#define DCMAX 0.00  /* max value of dc level w/r amp */
 

void GetPerturbation(float *y, int nh, int nd, float alpha, float amp,long *seed)
{
	int k,l,nxStorage,offset,n,nx;
	float dcLevel,maximum, dk, ab, c1, dummy;
	struct complex *x;
	float *tx;
	double pow(), sqrt(), exp();
	int i,j;
	
	nx = MAX(nh,nd);
	
	/* make sure nx is a power of two */
	n = 2;
	while(n < nx)
	{
		n *= 2;
	}
	
	nx = n;
	
	x  = (struct complex *) malloc(sizeof(struct complex)*nx*nx);
	/* RBH hack since cannot fill getrand(x,2*nx*nx,seed) because of omplex float */
	tx  = ( float *) malloc(sizeof( float)*2*nx*nx);
	
	/* READ 0 MEAN, 1 VARIANCE RANDOM ARRAY */
	/*
	getrand(tx,2*nx*nx,seed);
	*/
	for(i=0,j=0;i < nx*nx ; i++){
		x[i].re = tx[j++];
		x[i].im = tx[j++];
	}
	
	/* alpha *= 0.5; */
	dk = 2*PI/(float)nx;
	ab = (2.0 + alpha)*0.5;
	
	/* set up a DC level in the zero frequency */
	dcLevel = DCMAX * x[0].re * amp;
	
	x[0*nx+0].re = 0;
	x[0*nx+0].im = 0;
	/* ZERO THE NYQUIST */
	x[nx/2*nx+nx/2].re = 0.;
	x[nx/2*nx+nx/2].im = 0.;
	for(l=0;l<nx;l++) {
		x[l*nx+nx/2].re = 0.0;
		x[l*nx+nx/2].im = 0.0;
		x[nx*nx/2+l].re = 0.0;
		x[nx*nx/2+l].im = 0.0;
	}
	/* ADJUST TOP ROW AND LEFT COLUMN */
	for(l=1;l<nx/2;l++) {
		/* TOP */
		c1 = sqrt( dk*dk* exp(-alpha*alpha*l*l*dk*dk*0.5));
		x[0*nx+l].re *= c1;
		x[0*nx+l].im *= c1;
		x[0*nx+nx-l].re *= c1;
		x[0*nx+nx-l].im *= c1;
		/* LEFT */
		x[l*nx+0].re *= c1;
		x[l*nx+0].im *= c1;
		x[(nx-l)*nx+0].re *= c1;
		x[(nx-l)*nx+0].im *= c1;
	}
	/* ADJUST THE FOUR QUANDRANTS */
	for(k=1;k<nx/2;k++) {
		for(l=1;l<nx/2;l++) {
			c1 = sqrt( dk*dk* exp(-alpha*alpha*(l*l+k*k)*dk*dk*0.5));
			x[k*nx+l].re *= c1;
			x[k*nx+l].im *= c1;
			x[k*nx+nx-l].re *= c1;
			x[k*nx+nx-l].im *= c1;
			x[(nx-k)*nx+l].re *= c1;
			x[(nx-k)*nx+l].im *= c1;
			x[(nx-k)*nx+nx-l].re *= c1;
			x[(nx-k)*nx+nx-l].im *= c1;
		}
	}
	cfft2d(x,nx);


	/* window the upper left portion of the image */
	/* find max value, then normalize the perturbation to amp */
	
	offset = 0;
	for(l=0;l<nd;l++)
	{
	    for(k=0; k<nh; k++)
	    {
		dummy = x[(l+offset)*nx + (k+offset)].re;
		y[l * nh + k] = dummy;
	    }
	}

	free(x);

	/* go back to the original image size */
	
	dcLevel = 0;
	for(l=0;l<nh*nd;l++)
		dcLevel += y[l];

	for(l=0;l<nh*nd;l++)
		y[l] -= dcLevel;

	maximum = 0;
	for(l=0;l<nh*nd;l++)
		maximum = MAX(maximum,ABSOLUTE(y[l]));
	
	maximum /= amp;
	for(l=0;l<nh*nd;l++)
		y[l] = y[l]/maximum;
	
	return;
}

void getrand(float *x,int n, long *seed)
{
	int i, j;
	float dummy,sum=0,avg=0;
	float GetRandomNumber();

	/* GetRandomNumber returns a uniform
	   sequence between zero and one     */
	
	sum = 0;
	avg = 0;
	for(i=0;i<n;i++)
	 {
	   x[i] = GetRandomNumber(seed);
	   sum += x[i];
	 }

	 avg = sum/(float)n;
	 for(i=0;i<n;i++)
	    x[i] -= avg;
	 
	return;
}

void AddPerturbation(float *buffer1,float *buffer2, int Npts)
  {
  /* buffer1 = buffer1 + buffer2 */
  int i;
	for(i = 0; i < Npts; i++)
	   buffer1[i] += buffer2[i];
	return;
  }/* end AddPerturbation */



void CopyImage(float *buffer1,float *buffer2, int nelements)
  /* copy buffer2 into buffer1 */
  {
   int i;
   for(i = 0; i < nelements; i++)
      *buffer1++ = *buffer2++;
  }

