#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fcntl.h>
#include <sys/file.h>
#include "invsubs.h"

#define deg_to_rad 0.0174532
#define kVerySmall 0.000001

/* prototypes */
void WriteAModel(int mnumber,int iteration,float *coverage, float *s, float *v, float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x);

void WriteModels(int iteration, float *coverage, float *s, float *v, float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x, int nModels)
{
  int m;
  float *pC, *pS, *pV;
  for(m = 0; m < nModels;m++)
    {
      pC = &coverage[m*nx*ny];
      pS = &s[m*nx*ny];
      pV = &v[m*nx*ny];
      WriteAModel(m,iteration, pC, pS, pV, minlat, minlon, dx, dy, nx, ny, center_lon, min_x);
    }



}
/***********************************************************************************/
/*
 * write out the models into two ascii files one for the slow points, one for
 * the fast
 */
/***********************************************************************************/

void WriteAModel(int mnumber,int iteration,float *coverage, float *s, float *v, float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x)
{
	float           lat, lon, x, y, dc_velocity, sum;
	float           avg;
	int             i, j, k;
	FILE           *sfd, *ffd, *vcfd;
	char            ff[32], fs[32],fvc[32];

	sum = 0;
	j = 0;
	for (i = 0; i < nx * ny; i++) {
		if (fabs(s[i]) > kVerySmall)
			v[i] = 1 / s[i];
		else
			v[i] = 0;
		if (coverage[i] != 0) {
			sum += v[i];
			j += 1;
		}
	}
	
	sprintf(fvc, "vc_%3.3d_%3.3d.xyz",mnumber, iteration);
	vcfd = fopen(fvc, "w");
	for (i = 1; i <= ny; i += 1) {
		y = ((float) i - 0.5) * dy;
		lat = y / 111.19 + minlat;
		for (j = 1; j <= nx; j += 1) {
			x = ((float) j - 0.5) * dx;
			lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;

			k = (j - 1) + (i - 1) * nx;
			if (lon >= -180 && lon <= 180) {	/* apply a lon window
								 * because of the
								 * projection */
				fprintf(vcfd, "%.3f %.3f %.3f %.2f\n", lat, lon, v[k], coverage[k]);
			}
		}
	}
	fclose(vcfd);
	
	if (j != 0)
		sum /= (float) (j);

	/*  printf("Mean of nonzero correction velocities is: %.2f\n", sum); */

	avg = 0;
	for (i = 0; i < nx * ny; i++) {
		avg += v[i];
	}
	avg /= (float) (nx * ny);

	for (i = 0; i < nx * ny; i++) {
		v[i] = 100 * (v[i] - avg) / avg;
	}

/*
	sprintf(ff, "fast_%3.3d_%3.3d.txt",mnumber,iteration);
	sprintf(fs, "slow_%3.3d_%3.3d.txt", mnumber,iteration);
	sfd = fopen(fs, "w");
	ffd = fopen(ff, "w");

	for (i = 1; i <= ny; i += 1) {
		y = ((float) i - 0.5) * dy;
		lat = y / 111.19 + minlat;
		for (j = 1; j <= nx; j += 1) {
			x = ((float) j - 0.5) * dx;
			lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;

			k = (j - 1) + (i - 1) * nx;
			if (lon >= -180 && lon <= 180) {	
*/
								/* apply a lon window
								 * because of the
								 * projection */
/*
				if (v[k] <= 0.0)
					fprintf(sfd, "%f %f %f\n", lat, lon, v[k]);
				else
					fprintf(ffd, "%f %f %f\n", lat, lon, v[k]);
			}
		}
	}
	fclose(sfd);
	fclose(ffd);
*/
}


/***********************************************************************************/
/*
 * write out the models into two ascii files one for the slow points, one for
 * the fast
 */
/***********************************************************************************/

void WriteModelGrid(float minlat, float minlon, float dx, float dy, int nx, int ny, float center_lon, float min_x)
{
	float           lat, lon, x, y, dc_velocity, sum;
	float           avg;
	int             i, j, k;
	FILE           *sfd, *ffd, *vcfd, *invcfd;
	char            ff[32], fs[32],fvc[32];

	vcfd = fopen("modelGrid.xy", "w");
	invcfd = fopen("indexGrid.xy", "w");
	for (i = 1; i <= ny; i += 1) {
		for (j = 1; j <= nx; j += 1) {
                    y = ((float) i - 1) * dy;
                    lat = y / 111.19 + minlat;
                    x = ((float) j - 1) * dx;
                    lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;				 
                    fprintf(vcfd, "%.3f %.3f\n", lat, lon);
                    fprintf(invcfd, "%d %d %.3f %.3f\n", i, j, lat, lon);
                    
                    y += dy;
                    lat = y / 111.19 + minlat;
                    lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;
                    fprintf(vcfd, "%.3f %.3f\n", lat, lon);
                    
                    x += dx;
                    lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;				 
                    fprintf(vcfd, "%.3f %.3f\n", lat, lon);
                    
                    y -= dy;
                    lat = y / 111.19 + minlat;
                    lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;
                    fprintf(vcfd, "%.3f %.3f\n", lat, lon);
                    
                    x -= dx;
                    lon = (x - min_x) / (cos((lat) * deg_to_rad) * 111.19) + center_lon;				 
                    fprintf(vcfd, "%.3f %.3f\n>\n", lat, lon);
                                            
		}
	}
	fclose(vcfd);
	fclose(invcfd);
	
}
