      program doquad
c-----
c     this program reads entries from the tomography dispersion file output
c     which is of the form
c       LAT  LON  U   dU
c     and finds the 4 corners that surround or the desired disperion value
c     Then an interpolation is carried out to give the dispersion at the 
c     correct point
c-----
c     This program assumes that the tables are of the form
c     LAT  LON VEL dVEL   (or LAT LON VEL with -3 option)
c
c     The program ASSUMES that the user enters geographic
c     coordinates while the tables are in geodetic
c
c     A conversion is made from the geographic on the input to
c     geodetic to access the tables
c-----
       integer NSEG
       parameter(NSEG=1000)
       real xlat(NSEG), xlon(NSEG), dist(NSEG)
       real dkm
       logical dothree

      real lat1, lon1
      character fname*120

      logical doout, lnocell

      call gcmdln(geoglat,geoglon,fname,lnocell,dothree)

C      WRITE(0,*)lat1,lon1,fname
c
      call geog_to_geoc(geoglat,geoglon,geoclat,geoclon)
      dkm = 50.0
      dl=50.0
      open(1,file=fname,access='sequential',
     1 form='formatted',status='unknown')
      rewind 1
      sumd = 0.0
      sumt = 0.0
      doout = .true.
   
          call getu(geoclat,geoclon,wll,wlr,wur,wul,u,dothree)
c-----
c      only interpolate if all 4 corners have total ray length >= dl km
c-----
          if(wll.le.dl .or. wlr.le.dl .or. 
     1              wur.lt.dl .or. wul.lt.dl)then
               doout = .false.
          endif
Cold         sumd = sumd + dist(i)
Cold         sumt = sumt + dist(i)/u
      if(doout .or. lnocell)then
           write(6,'(f10.4)')u
      endif
      close(1)
      end

      subroutine gcmdln(lat1,lon1,fname,lnocell,dothree)
      real lat1,lon1
      character fname*(*)
      logical lnocell,dothree
c-----
c     lnocell     L    if true use the cell irrespective of sampling
c-----

      character names*80
      integer mnmarg
      integer i, nmarg

      lnocell = .false.
      dothree = .false.
      lat1 = 0
      lon1 = 0
      i = 0
      nmarg = mnmarg()
 1000 continue
           i = i + 1
           if(i.gt.nmarg)go to 2000
           call mgtarg(i,names)
           if(names(1:4).eq.'-LAT')then
               i = i + 1
               call mgtarg(i,names)
               read(names,'(bn,f10.0)')lat1
           else if(names(1:4).eq.'-LON')then
               i = i + 1
               call mgtarg(i,names)
               read(names,'(bn,f10.0)')lon1
           else if(names(1:2).eq.'-F')then
               i = i + 1
               call mgtarg(i,fname)
           else if(names(1:2).eq.'-C')then
               lnocell = .true.
           else if(names(1:2).eq.'-3')then
               dothree = .true.
           else if(names(1:2).eq.'-h')then
               call usage()
           endif
           go to 1000
 2000 continue
      if(nmarg.lt.6 .or.nmarg.gt.8)call usage()
      return
      end

      subroutine usage()
      integer LER
      parameter(LER=0)
      write(LER,*)'Usage: doquad -LAT lat -LON lon',
     1 '  -F filename -h [-3]'
      write(LER,*)
     1 ' Interp[olate grid results to give dispersion for LAT,LON'
      write(LER,*)
     1 ' -LAT lat  (no default) geographic latitude'
      write(LER,*)
     1 ' -LON lon  (no default) geographic longitude'
      write(LER,*)
     1 ' -3        (no default) Normally read lat,lon,v,dv'
      write(LER,*)
     1 '                        from tables. With -3 read lat,lon,v'
      write(LER,*)
     1 ' -C        (no default) Debug: output results for empty cells'
      stop
      end

      subroutine getu(lat,lon,wll,wlr,wur,wul,u,dothree)
      real lat,lon,wll,wlr,wur,wul,u
      logical dothree
c-----
c      initialize
c-----
      xur = -12345
      yur = -12345
      distur = 1.0e+30
      xul =  12345
      yul = -12345
      distul = 1.0e+30
      xlr = -12345
      ylr =  12345
      distlr = 1.0e+30
      xll =  12345
      yll =  12346
      distll = 1.0e+30
c-----
c     read in the values
c-----
      degrad = 3.1415927/180.0
c-----
c     LAT LON VELOCITY TOTAL_LENGTH_RAYS_IN_CELL
      rewind 1
 1000 if(dothree)then
         read(1,*,end=9999)y,x,u
         du = 100
      else
         read(1,*,end=9999)y,x,u,du
      endif
      
      dist = sqrt((y-lat)**2 
     1    + (cos(0.5*degrad*(y+(lat)))*(x-(lon)))**2)
c-----
c     focus on ur
c-----
      if(y.gt.lat .and. x.gt.lon .and. dist.lt.distur)then
            distur = dist
            yur = y
            xur = x
            uur = u
            wur = du
      endif
      if(y.gt.lat .and. x.le.lon .and. dist.lt.distul)then
            distul = dist
            yul = y
            xul = x
            uul = u
            wul = du
      endif
      if(y.le.lat .and. x.gt.lon .and. dist.lt.distlr)then
            distlr = dist
            ylr = y
            xlr = x
            ulr = u
            wlr = du
      endif
      if(y.le.lat .and. x.le.lon .and. dist.lt.distll)then
            distll = dist
            yll = y
            xll = x
            ull = u
            wll = du
      endif
      go to 1000
 9999 continue
C      WRITE(0,*)'dointquad',lat,lon
C      WRITE(0,*)'UL:',yul,xul,uul
C      WRITE(0,*)'UR:',yur,xur,uur
C      WRITE(0,*)'LL:',yll,xll,ull
C      WRITE(0,*)'LR:',ylr,xlr,ulr
c-----
c     define the corners
c-----
c     use the 4 corners to interpolate
c     first interpolate on longitude and then latitude
c     the region is a quadrilateral but not a rectangle
c     Colorado has a lt,lon grid
c     Ammon has a constant latitude but longitude sampling changes with latitude
c     This is the reason for interpolating on longitude first
c     For the interpolation x is longitude and y is latitude
c-----
c     do the top first
c-----
      px = ( lon - xul)/(xur - xul)
      ytop = (1.0-px)*yul + px*yur
      utop = (1.0-px)*uul + px*uur
C      write(6,*)'TOP',ytop,-90,utop
c-----
c     do the bottom
c-----
      px = ( lon - xll)/(xlr - xll)
      ybot = (1.0-px)*yll + px*ylr
      ubot = (1.0-px)*ull + px*ulr
C      write(6,*)'BOT',ybot,-90,ubot
c-----
c     now interpolate on latitude
c-----
      py = (lat - ybot)/(ytop - ybot)
C      WRITE(6,*)'PY:',py
      u = (1.0-py)*ubot + py*utop
      return
      end
        function mnmarg()
c---------------------------------------------------------------------c
c                                                                     c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                c
c      VOLUME V                                                       c
c                                                                     c
c      SUBROUTINE: MNMARG                                             c
c                                                                     c
c      COPYRIGHT 1996 R. B. Herrmann                                  c
c                                                                     c
c      Department of Earth and Atmospheric Sciences                   c
c      Saint Louis University                                         c
c      221 North Grand Boulevard                                      c
c      St. Louis, Missouri 63103                                      c
c      U. S. A.                                                       c
c                                                                     c
c---------------------------------------------------------------------c
        implicit none
        integer iargc
        integer mnmarg
            mnmarg = iargc()
        return
        end

        subroutine mgtarg(i,name)
c---------------------------------------------------------------------c
c                                                                     c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                c
c      VOLUME V                                                       c
c                                                                     c
c      SUBROUTINE: MGTARG                                             c
c                                                                     c
c      COPYRIGHT 1996 R. B. Herrmann                                  c
c                                                                     c
c      Department of Earth and Atmospheric Sciences                   c
c      Saint Louis University                                         c
c      221 North Grand Boulevard                                      c
c      St. Louis, Missouri 63103                                      c
c      U. S. A.                                                       c
c                                                                     c
c---------------------------------------------------------------------c
c-----
c       return the i'th command line argument
c
c       This version works on SUN, IBM RS6000
c-----
        implicit none
        integer i
        character name*(*)
            call getarg(i,name)
        return
        end

c-----
c     online tool at http://www.apsalin.com/convert-geodetic-to-cartesian.aspx
c     and then look to xyz to get desired geocentric angle
c-----
c     Geodetic Coordinate Conversions
c     James R. Clynch
c     Naval Postgraduate School, 2002
c     www.oc.nps.edu/oc2902w/coord/coordcvt.pdf
c-----
c     https://en.wikipedia.org/wiki/Latitude
c     f = flattening = (a-b)/b where b = polar radius and a = equatorial radius
c     e^2 = 2f - f^2
c     b = a(1-f) = a(sqrt(1-e^2)
c     WGS84 ellipsoid
c     a (equatorial radius): 6378137.0 m exactly
c     1/f =  (inverse flattening): 298.257223563 exactly
c       from which
c     b (polar radius): 6356752.3142 m
c     e^2 (eccentricity squared): 0.00669437999014
c     tan lat_geocentric = (1 - e^2) tan lat_geodetic
c-----
c
c     From PROGRAMS.330/VOLI/src/udelaz.c
C   double degrad = 0.0174532925199433;     /* degrees to radians */
C   double a = 6378.137 ;                   /* equatorial radius */
C   double b = 6356.752 ;                   /* polar radius = a(1-f) */
C   double f = 0.003352810664747481 ;       /* f = 1 - b/a = 
C                                                    1/298.257223563 */
C   double e = 0.081819191 ;                        /* eccentricity (A.38)
C                                                   e^2 = (a^2 - b^2)/a^2
C                                                   e^2 = 2f - f^2 */
C   double e2 = 0.006694379990141317 ;      /* e^2 */
C   double eps = 0.006739496742276435 ;     /* second ellipticity (A.42)
C                                                   eps = (a^2 - b^2)/b^2
C                                                   = e^2/(1-e^2) */
C   static float RE=6371.003;
C           c = cos(degrad*lat);
C           s = sin(degrad*lat);
C           e4 = (1.-e2)*(1.-e2);
C           fac = sqrt(e4 + (1.0-e4)*c*c);
C           *slat = (1.-e2) * s /fac;
C           *clat =      c /fac;
C   
c-----
      subroutine geog_to_geoc(geoglat,geoglon,geoclat,geoclon)
      real geoglat,geoglon,geoclat,geoclon
      double precision degrad, a, b, f, e, e2, eps, RE, e4, fac
      double precision c, s, clat, slat, dang
c-----
c     define constants
c-----
      e2 = 0.006694379990141317
      degrad = 0.0174532925199433

c-----
c     longitude does not change
c-----
      geoclon = geoglon
c-----
c     the conversion formula is typically given in terms of tan
c     by forming cos and sin and normalizing so that cos^2 + sin^2 = 1
c     we can use the atan2 to get proper angle in radians
c-----
      c = dcos(degrad*geoglat)
      s = dsin(degrad*geoglat)
      e4 = (1.-e2)*(1.-e2)
      fac = sqrt(e4 + (1.0-e4)*c*c)
      slat = (1.-e2) * s /fac
      clat =           c /fac
      geoclat = datan2(slat,clat)/degrad
      return
      end
      

      subroutine geoc_to_geog(geoclat,geoclon,geoglat,geoglon)
      real geoglat,geoglon,geoclat,geoclon
      double precision degrad, a, b, f, e, e2, eps, RE, e4, fac
      double precision c, s, clat, slat, dang
c-----
c     define constants
c-----
      e2 = 0.006694379990141317
      degrad = 0.0174532925199433

c-----
c     longitude does not change
c-----
      geoglon = geoclon
c-----
c     the conversion formula is typically given in terms of tan
c     by forming cos and sin and normalizing so that cos^2 + sin^2 = 1
c     we can use the atan2 to get proper angle in radians
c-----
      c = dcos(degrad*geoclat)
      s = dsin(degrad*geoclat)
      e4 = (1.-e2)*(1.-e2)
      fac = sqrt( (s/(1-e2))**2 + c**2)
      slat =           s /(fac*(1-e2))
      clat =           c /(fac)
      geoglat = datan2(slat,clat)/degrad
      return
      end
      

